'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var produce = _interopDefault(require('immer'));
var React = require('react');
var React__default = _interopDefault(React);
var useContextSelector = require('use-context-selector');

function shallowEqual(a, b) {
  if (a === b) {
    return true;
  }

  if (typeof a !== 'object' || typeof b !== 'object') {
    return false;
  }

  if (a === null || b === null) {
    return false;
  }

  if (Object.keys(a).length !== Object.keys(b).length) {
    return false;
  }

  for (var key in a) {
    if (!(key in b)) {
      return false;
    }

    if (a[key] !== b[key]) {
      return false;
    }
  }

  return true;
}

function createKit(_ref) {
  var initialState = _ref.initialState,
      reducers = _ref.reducers;
  var Context = useContextSelector.createContext(null);

  var Provider = function Provider(_ref2) {
    var children = _ref2.children;

    function reducerWithImmer(reducers) {
      if (reducers === void 0) {
        reducers = {};
      }

      return function combination(state, action) {
        var next = produce(state, function (draft) {
          var key = action.type;
          reducers[key](draft, action);
        });
        return next;
      };
    }

    var _useReducer = React.useReducer(reducerWithImmer(reducers), initialState),
        state = _useReducer[0],
        dispatch = _useReducer[1];

    var snapshots = React.useRef([]);
    return React__default.createElement(Context.Provider, {
      value: [state, dispatch, snapshots]
    }, children);
  };

  function eqlFn(prevValue, nextValue) {
    var r = prevValue === nextValue || shallowEqual(prevValue, nextValue);
    return r;
  }

  function useSelector(selector) {
    var patchedSelector = React.useMemo(function () {
      var prevValue = null;
      return function (_ref3) {
        var state = _ref3[0];
        var nextValue = selector(state);

        if (eqlFn(prevValue, nextValue)) {
          return prevValue;
        }

        return prevValue = nextValue;
      };
    }, [selector]);
    return useContextSelector.useContextSelector(Context, patchedSelector);
  }

  function useDispatch() {
    return useContextSelector.useContextSelector(Context, function (v) {
      return v[1];
    });
  }

  return {
    Context: Context,
    Provider: Provider,
    useSelector: useSelector,
    useDispatch: useDispatch
  };
}

exports.createKit = createKit;
//# sourceMappingURL=reducer-kit.cjs.development.js.map
